@@ -20,12 +20,29 @@ pub enum MessageType {
     Signal = 4,
 }
 
+impl<'a> TryFrom<&'a str> for MessageType {
+    type Error = ();
+
+    fn try_from(value: &'a str) -> Result<Self, <crate::message::MessageType as TryFrom<&'a str>>::Error> {
+        match value {
+            "error" => Ok(MessageType::Error),
+            "method_call" => Ok(MessageType::MethodCall),
+            "method_return" => Ok(MessageType::MethodReturn),
+            "signal" => Ok(MessageType::Signal),
+            _ => Err(())
+        }
+    }
+}
+
 mod signalargs;
 pub use self::signalargs::SignalArgs;
 
 mod matchrule;
 pub use self::matchrule::MatchRule;
+use std::convert::TryFrom;
 
+mod parser;
+pub use self::parser::Error as MatchRuleParserError;
 
 /// A D-Bus message. A message contains headers - usually destination address, path, interface and member,
 /// and a list of arguments.
@@ -344,7 +361,7 @@ impl Message {
         Ok(R::read(&mut self.iter_init())?)
     }
 
-    /// Returns a struct for retreiving the arguments from a message. Supersedes get_items().
+    /// Returns a struct for retrieving the arguments from a message. Supersedes get_items().
     pub fn iter_init(&self) -> Iter { Iter::new(&self) }
 
     /// Gets the MessageType of the Message.
@@ -434,6 +451,15 @@ impl Message {
         unsafe { ffi::dbus_message_set_serial(self.msg, val) };
     }
 
+    /// Sets the object path of this message - you should not use this method
+    ///
+    /// Every message has a path already, and there is no good reason to change it using this method.
+    pub fn set_path(&mut self, path: &Path)  {
+        if unsafe { !ffi::dbus_message_set_path(self.msg, path.as_ptr()) } {
+            panic!("out of memory");
+        }
+    }
+
     /// Marshals a message - mostly for internal use
     ///
     /// The function f will be called one or more times with bytes to be written somewhere.
@@ -472,6 +498,15 @@ impl Message {
         if x < MIN_HEADER as _ { Err(()) } else { Ok(x as usize) }
     }
 
+    /// Sets sender manually - mostly for internal use
+    ///
+    /// When sending a message, a sender will be automatically assigned, so you don't need to call
+    /// this method. However, it can be very useful in test code that is supposed to handle a method call.
+    /// This way, you can create a method call and handle it without receiving it from a real D-Bus instance.
+    pub fn set_sender(&mut self, sender: Option<BusName>) {
+        let c_sender = sender.as_ref().map(|d| d.as_cstr().as_ptr()).unwrap_or(ptr::null());
+        assert!(unsafe { ffi::dbus_message_set_sender(self.msg, c_sender) } != 0);
+    }
 }
 
 impl Drop for Message {
@@ -522,6 +557,16 @@ mod test {
         assert!(m.get_no_reply());
     }
 
+    #[test]
+    fn set_valid_sender() {
+        let mut m = Message::new_method_call("org.test.rust", "/", "org.test.rust", "Test").unwrap();
+        let sender = ":1.14";
+        let d = Some(BusName::new(sender).unwrap());
+        m.set_sender(d);
+
+        assert_eq!(sender, m.sender().unwrap().to_string());
+    }
+
     #[test]
     fn marshal() {
         let mut m = Message::new_method_call("org.freedesktop.DBus", "/org/freedesktop/DBus", "org.freedesktop.DBus", "Hello").unwrap();
